"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.EpicStripePaymentsSubscription = void 0;
const exceptions_1 = require("../../exceptions");
const EpicStripePaymentsSubscription = (Parent) => class {
    constructor(configuration) {
        // Create SDK Instance
        this.Instance = Parent.createInstance();
        this.configure = (configuration) => {
            if (typeof configuration === "object") {
                if (!configuration.returnUrl ||
                    typeof configuration.returnUrl !== "string")
                    // Validate Configuration
                    throw new exceptions_1.GatewayException(`Please provide a valid Return URL!`);
                if (!configuration.cancelUrl ||
                    typeof configuration.cancelUrl !== "string")
                    throw new exceptions_1.GatewayException(`Please provide a valid Cancel URL!`);
                // Save Configuration
                this.Configuration = configuration;
            }
            return this;
        };
        this.create = (Items, custom) => new Promise((resolve, reject) => {
            var _a;
            // Get Items Output
            const Output = Items.get("Subscription");
            // Create Payment Session
            this.Instance.checkout.sessions
                .create(Object.assign({ mode: "subscription", payment_method_types: ((_a = this.Configuration) === null || _a === void 0 ? void 0 : _a.methods) || ["card"], success_url: this.Configuration.returnUrl, cancel_url: this.Configuration.cancelUrl, line_items: Output.items.map((item) => (Object.assign(Object.assign({}, item), { sku: undefined, name: undefined, currency: undefined }))) }, custom))
                .then((response) => {
                var _a, _b, _c, _d;
                return resolve({
                    method: Parent.getMethod(),
                    mode: ((_a = Parent.getConfiguration()) === null || _a === void 0 ? void 0 : _a.mode) || "Test",
                    key: (_d = (_b = Parent.getConfiguration()) === null || _b === void 0 ? void 0 : _b.credentials[((_c = Parent.getConfiguration()) === null || _c === void 0 ? void 0 : _c.mode) || "Test"]) === null || _d === void 0 ? void 0 : _d.PK,
                    response,
                });
            })
                .catch((error) => reject(new exceptions_1.GatewayException(error)));
        });
        this.execute = (options) => new Promise(async (resolve, reject) => {
            var _a, _b, _c, _d, _e;
            try {
                // Resolve Checkout Mode
                const Mode = ((_a = Parent.getConfiguration()) === null || _a === void 0 ? void 0 : _a.mode) || "Test";
                // Webhook Secret
                const Secret = (_d = (_c = (_b = Parent.getConfiguration()) === null || _b === void 0 ? void 0 : _b.credentials) === null || _c === void 0 ? void 0 : _c[Mode]) === null || _d === void 0 ? void 0 : _d.WebhookSecret;
                // Check If Webhook Secret Provided
                if (typeof Secret === "string")
                    options.body = this.Instance.webhooks.constructEvent(options.body, options.signature || "", Secret);
                // Check Events
                switch (options.body.type) {
                    case "invoice.paid":
                        resolve({
                            method: Parent.getMethod(),
                            mode: ((_e = Parent.getConfiguration()) === null || _e === void 0 ? void 0 : _e.mode) || "Test",
                            response: {
                                invoice: options.body.data.object,
                                payment_intent: await this.Instance.paymentIntents.retrieve(options.body.data.object.payment_intent),
                            },
                        });
                        break;
                    case "invoice.payment_failed":
                        throw new Error(options.body);
                    default:
                        throw new Error(`Invalid Event Type '${options.body.type}' has been encountered!`);
                }
            }
            catch (error) {
                reject(new exceptions_1.GatewayException(error));
            }
        });
        this.configure(configuration);
    }
};
exports.EpicStripePaymentsSubscription = EpicStripePaymentsSubscription;
